@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -69,11 +41,16 @@ import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Calendar;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.LinkedHashMap;
+import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.Properties;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.CommandLineParser;
 import org.apache.commons.cli.HelpFormatter;
@@ -99,6 +76,7 @@ import com.openexchange.java.Streams;
  *
  * @author <a href="mailto:karsten.will@open-xchange.com">Karsten Will</a>
  */
+@SuppressWarnings("deprecation")
 public class Datamining {
 
     /**
@@ -114,8 +92,6 @@ public class Datamining {
 
     public static final String NUMBER_OF_SCHEMATA = "numberOfSchemata";
 
-    private static Connection configdbConnection = null;
-
     private static String configDBURL = "";
 
     private static String configDBUser = "";
@@ -132,13 +108,15 @@ public class Datamining {
 
     private static String filename = "";
 
-    private static StringBuilder reportStringBuilder = new StringBuilder();
+    private static final AtomicBoolean noTimeout = new AtomicBoolean(true);
 
-    public static ArrayList<String> allTheQuestions = new ArrayList<String>();
+    private static final StringBuilder reportStringBuilder = new StringBuilder();
 
-    private static HashMap<String, String> allTheAnswers = new HashMap<String, String>();
+    public static final List<String> allTheQuestions = new ArrayList<String>();
 
-    private static ArrayList<Schema> allSchemata;
+    private static final HashMap<String, String> allTheAnswers = new HashMap<String, String>();
+
+    private static List<Schema> allSchemata;
 
     private static Options staticOptions;
 
@@ -155,81 +133,93 @@ public class Datamining {
                 printHelp();
             }
         } else if (!helpPrinted) {
-            configdbConnection = getDBConnection(configDBURL, configDBUser, configDBPassword);
-            allSchemata = getAllSchemata();
-            report(NUMBER_OF_SCHEMATA, Integer.toString(allSchemata.size()));
+            Connection configdbConnection = getDBConnection(configDBURL, configDBUser, configDBPassword);
+            try {
+                allSchemata = getAllSchemata(configdbConnection);
+                report(NUMBER_OF_SCHEMATA, Integer.toString(allSchemata.size()));
 
-            reportAverageFilestoreSize();
-            reportNumberOfContexts();
-            Questions.reportNumberOfUsers();
-            Questions.reportNumberOfAppointments();
-            Questions.reportNumberOfUsersWhoCreatedAppointments();
-            Questions.reportMaximumNumberOfCreatedAppointmentsForOneUser();
-            Questions.reportAverageNumberOfAppointmentsPerUserWhoHasAppointmentsAtAll();
-            Questions.reportNumberOfUsersWithEventsInPrivateCalendarThatAreInTheFutureAndAreNotYearlySeries();
-            Questions.reportNumberOfUsersWhoChangedTheirCalendarInTheLast30Days();
-            Questions.reportNumberOfDocuments();
-            Questions.reportNumberOfUsersWhoCreatedDocuments();
-            Questions.reportMaximumNumberOfCreatedDocumentsForOneUser();
-            Questions.reportAverageNumberOfDocumentsPerUserWhoHasDocumentsAtAll();
-            reportAverageNumberOfInfostoreObjectsPerContext();
-            reportAverageNumberOfInfostoreObjectsPerSchema();
-            Questions.reportNumberOfNewInfostoreObjectsInTheLast30Days();
-            Questions.reportNumberOfChangedInfostoreObjectsInTheLast30Days();
-            Questions.reportNumberOfUsersWithNewInfostoreObjectsInTheLast30Days();
-            Questions.reportSliceAndDiceOnDocumentSize();
-            Questions.reportAverageDocumentSize();
-            Questions.reportNumberOfContacts();
-            Questions.reportNumberOfUserCreatedContacts();
-            Questions.reportNumberOfUsersWhoHaveContacts();
-            Questions.reportNumberOfUsersWhoCreatedContacts();
-            Questions.reportMaximumNumberOfContactsForOneUser();
-            Questions.reportMaximumNumberOfCreatedContactsForOneUser();
-            Questions.reportAverageNumberOfContactsPerUserWhoHasContactsAtAll();
-            Questions.reportAverageNumberOfContactsPerUserWhoHasCreatedContacts();
-            Questions.reportNumberOfUsersWhoChangedTheirContactsInTheLast30Days();
-            Questions.reportNumberOfUsersWithLinkedSocialNetworkingAccounts();
-            Questions.reportNumberOfUsersConnectedToLinkedIn();
-            Questions.reportNumberOfUsersConnectedToTwitter();
-            Questions.reportNumberOfUsersConnectedToGoogle();
-            Questions.reportNumberOfUsersConnectedToMSN();
-            Questions.reportNumberOfUsersConnectedToYahoo();
-            Questions.reportNumberOfUsersConnectedToXing();
-            Questions.reportNumberOfUsersConnectedToTOnline();
-            Questions.reportNumberOfUsersConnectedToGMX();
-            Questions.reportNumberOfUsersConnectedToWebDe();
-            Questions.reportNumberOfTasks();
-            Questions.reportNumberOfUsersWhoCreatedTasks();
-            Questions.reportMaximumNumberOfCreatedTasksForOneUser();
-            Questions.reportAverageNumberOfTasksPerUserWhoHasTasksAtAll();
-            Questions.reportNumberOfUsersWhoChangedTheirTasksInTheLast30Days();
-            Questions.reportNumberOfUsersWhoSelectedTeamViewAsCalendarDefault();
-            Questions.reportNumberOfUsersWhoSelectedCalendarViewAsCalendarDefault();
-            Questions.reportNumberOfUsersWhoSelectedListViewAsCalendarDefault();
-            Questions.reportNumberOfUsersWhoSelectedCardsViewAsContactsDefault();
-            Questions.reportNumberOfUsersWhoSelectedListViewAsContactsDefault();
-            Questions.reportNumberOfUsersWhoSelectedListViewAsTasksDefault();
-            Questions.reportNumberOfUsersWhoSelectedHSplitViewAsTasksDefault();
-            Questions.reportNumberOfUsersWhoSelectedListViewAsInfostoreDefault();
-            Questions.reportNumberOfUsersWhoSelectedHSplitViewAsInfostoreDefault();
-            Questions.reportNumberOfUsersWhoActivatedMiniCalendar();
-            Questions.reportNumberOfUsersWhoLoggedInWithClientOX6UIInTheLast30Days();
-            Questions.reportNumberOfUsersWhoLoggedInWithClientAppSuiteUIInTheLast30Days();
-            Questions.reportNumberOfUsersWhoLoggedInWithClientMobileUIInTheLast30Days();
-            Questions.reportNumberOfUsersWhoLoggedInWithClientEASInTheLast30Days();
-            Questions.reportNumberOfUsersWhoLoggedInWithClientCalDAVInTheLast30Days();
-            Questions.reportNumberOfUsersWhoLoggedInWithClientCardDAVInTheLast30Days();
-            Questions.reportSliceAndDiceOnDraftMailSize();
-            Questions.reportSliceAndDiceOnExternalAccountUsage();
+                reportAverageFilestoreSize(configdbConnection);
+                reportNumberOfContexts(configdbConnection);
 
-            rightNow = Calendar.getInstance();
-            final long after = rightNow.getTime().getTime();
-            report("durationInSeconds", Long.toString((after - before) / 1000));
+                closeSafely(configdbConnection);
+                configdbConnection = null;
 
-            sanityCheck();
+                Questions.reportNumberOfUsers();
+                Questions.reportNumberOfAppointments();
+                Questions.reportNumberOfUsersWhoCreatedAppointments();
+                Questions.reportMaximumNumberOfCreatedAppointmentsForOneUser();
+                Questions.reportAverageNumberOfAppointmentsPerUserWhoHasAppointmentsAtAll();
+                Questions.reportNumberOfUsersWithEventsInPrivateCalendarThatAreInTheFutureAndAreNotYearlySeries();
+                Questions.reportNumberOfUsersWhoChangedTheirCalendarInTheLast30Days();
+                Questions.reportNumberOfDocuments();
+                Questions.reportNumberOfUsersWhoCreatedDocuments();
+                Questions.reportMaximumNumberOfCreatedDocumentsForOneUser();
+                Questions.reportAverageNumberOfDocumentsPerUserWhoHasDocumentsAtAll();
+                reportAverageNumberOfInfostoreObjectsPerContext();
+                reportAverageNumberOfInfostoreObjectsPerSchema();
+                Questions.reportNumberOfNewInfostoreObjectsInTheLast30Days();
+                Questions.reportNumberOfChangedInfostoreObjectsInTheLast30Days();
+                Questions.reportNumberOfUsersWithNewInfostoreObjectsInTheLast30Days();
+                Questions.reportSliceAndDiceOnDocumentSize();
+                Questions.reportAverageDocumentSize();
+                Questions.reportNumberOfContacts();
+                Questions.reportNumberOfUserCreatedContacts();
+                Questions.reportNumberOfUsersWhoHaveContacts();
+                Questions.reportNumberOfUsersWhoCreatedContacts();
+                Questions.reportMaximumNumberOfContactsForOneUser();
+                Questions.reportMaximumNumberOfCreatedContactsForOneUser();
+                Questions.reportAverageNumberOfContactsPerUserWhoHasContactsAtAll();
+                Questions.reportAverageNumberOfContactsPerUserWhoHasCreatedContacts();
+                Questions.reportNumberOfUsersWhoChangedTheirContactsInTheLast30Days();
+                Questions.reportNumberOfUsersWithLinkedSocialNetworkingAccounts();
+                Questions.reportNumberOfUsersConnectedToLinkedIn();
+                Questions.reportNumberOfUsersConnectedToTwitter();
+                Questions.reportNumberOfUsersConnectedToGoogle();
+                Questions.reportNumberOfUsersConnectedToMSN();
+                Questions.reportNumberOfUsersConnectedToYahoo();
+                Questions.reportNumberOfUsersConnectedToXing();
+                Questions.reportNumberOfUsersConnectedToTOnline();
+                Questions.reportNumberOfUsersConnectedToGMX();
+                Questions.reportNumberOfUsersConnectedToWebDe();
+                Questions.reportNumberOfTasks();
+                Questions.reportNumberOfUsersWhoCreatedTasks();
+                Questions.reportMaximumNumberOfCreatedTasksForOneUser();
+                Questions.reportAverageNumberOfTasksPerUserWhoHasTasksAtAll();
+                Questions.reportNumberOfUsersWhoChangedTheirTasksInTheLast30Days();
+                Questions.reportNumberOfUsersWhoSelectedTeamViewAsCalendarDefault();
+                Questions.reportNumberOfUsersWhoSelectedCalendarViewAsCalendarDefault();
+                Questions.reportNumberOfUsersWhoSelectedListViewAsCalendarDefault();
+                Questions.reportNumberOfUsersWhoSelectedCardsViewAsContactsDefault();
+                Questions.reportNumberOfUsersWhoSelectedListViewAsContactsDefault();
+                Questions.reportNumberOfUsersWhoSelectedListViewAsTasksDefault();
+                Questions.reportNumberOfUsersWhoSelectedHSplitViewAsTasksDefault();
+                Questions.reportNumberOfUsersWhoSelectedListViewAsInfostoreDefault();
+                Questions.reportNumberOfUsersWhoSelectedHSplitViewAsInfostoreDefault();
+                Questions.reportNumberOfUsersWhoActivatedMiniCalendar();
+                Questions.reportNumberOfUsersWhoLoggedInWithClientOX6UIInTheLast30Days();
+                Questions.reportNumberOfUsersWhoLoggedInWithClientAppSuiteUIInTheLast30Days();
+                Questions.reportNumberOfUsersWhoLoggedInWithClientMobileUIInTheLast30Days();
+                Questions.reportNumberOfUsersWhoLoggedInWithClientEASInTheLast30Days();
+                Questions.reportNumberOfUsersWhoLoggedInWithClientCalDAVInTheLast30Days();
+                Questions.reportNumberOfUsersWhoLoggedInWithClientCardDAVInTheLast30Days();
+                Questions.reportSliceAndDiceOnDraftMailSize();
+                Questions.reportSliceAndDiceOnExternalAccountUsage();
 
-            printReport();
+                rightNow = Calendar.getInstance();
+                final long after = rightNow.getTime().getTime();
+                report("durationInSeconds", Long.toString((after - before) / 1000));
 
+                sanityCheck();
+
+                printReport();
+            } finally {
+                closeSafely(configdbConnection);
+            }
+        }
+    }
+
+    private static void closeSafely(Connection configdbConnection) {
+        if (configdbConnection != null) {
             try {
                 configdbConnection.close();
             } catch (SQLException e) {
@@ -278,7 +268,7 @@ public class Datamining {
     private static void printHelp() {
             HelpFormatter formatter = new HelpFormatter();
             formatter.setWidth(120);
-            formatter.printHelp( "datamining", staticOptions );
+            formatter.printHelp("datamining", staticOptions);
             System.out.println("\nEither specify these parameters manually or use this tool on an Open-Xchange application server where all necessary info\n is automatically found in the file /opt/open-xchange/etc/configdb.properties");
             System.out.println("Even then it is possible to override parameters from configdb.properties by setting them manually.");
             System.out.println("Please note that if you want to override hostname / dbName / dbPort at least hostname must be given (no default) and defaults will apply for the other 2.");
@@ -286,7 +276,7 @@ public class Datamining {
             helpPrinted = true;
     }
 
-    private static void reportNumberOfContexts() {
+    private static void reportNumberOfContexts(Connection configdbConnection) {
         allTheQuestions.add(NUMBER_OF_CONTEXTS);
         if (configdbConnection != null) {
             Statement query = null;
@@ -330,6 +320,7 @@ public class Datamining {
     	options.addOption(OptionBuilder.withLongOpt("verbose").withDescription("Print the results to the console as well as into the report file").create("v"));
     	options.addOption(OptionBuilder.withLongOpt("dbPort").hasArg().withDescription("Port where MySQL is running on the host specified with \"-hostname\" (default: \"3306\")").create("o"));
     	options.addOption(OptionBuilder.withLongOpt("help").withDescription("Print this helpfile").create("h"));
+    	options.addOption(OptionBuilder.hasArg(false).withLongOpt("timeout").withDescription("If this flag is set connections to database are supposed to be established with timeout settings. If flag is absent connections are established without timeout settings.").create("t"));
 
     	staticOptions = options;
 
@@ -362,6 +353,10 @@ public class Datamining {
 				verbose = true;
 			}
 
+			if (cl.hasOption("timeout")){
+			    noTimeout.set(false);
+			}
+
 			if (cl.hasOption("help")){
 				printHelp();
 			}
@@ -388,12 +383,17 @@ public class Datamining {
             configDBPassword = configdbProperties.getProperty("readProperty.2").substring(9);
 
             if (configDBURL != null && configDBUser != null && configDBPassword != null) {
-                System.out.println("All necessary parameters were found in /opt/open-xchange/etc/configdb.properties");
+                System.out.println("All necessary parameters found in '/opt/open-xchange/etc/configdb.properties' file");
             }
         } catch (FileNotFoundException e) {
-            System.out.println("File /opt/open-xchange/etc/configdb.properties is not available");
+            System.out.println("File '/opt/open-xchange/etc/configdb.properties' is not available");
+            System.out.println("Aborting...");
+            System.exit(1);
         } catch (IOException e) {
-            System.out.println("File /opt/open-xchange/etc/configdb.properties is not readable");
+            System.out.println("File '/opt/open-xchange/etc/configdb.properties' is not readable: " + e.getMessage());
+            e.printStackTrace(System.out);
+            System.out.println("Aborting...");
+            System.exit(1);
         } finally {
             if (null != in) {
                 try {
@@ -407,6 +407,9 @@ public class Datamining {
         // Try to read dbconnector.yaml
         in = null;
         try {
+            // Whether to consider timeout settings
+            boolean applyTimeout = noTimeout.get() == false;
+
             // Set defaults:
             Properties jdbcProps = new Properties();
             jdbcProps.setProperty("useUnicode", "true");
@@ -415,9 +418,15 @@ public class Datamining {
             jdbcProps.setProperty("useServerPrepStmts", "false");
             jdbcProps.setProperty("useTimezone", "true");
             jdbcProps.setProperty("serverTimezone", "UTC");
-            jdbcProps.setProperty("connectTimeout", "15000");
-            jdbcProps.setProperty("socketTimeout", "15000");
+            if (applyTimeout) {
+                jdbcProps.setProperty("connectTimeout", "15000");
+                jdbcProps.setProperty("socketTimeout", "15000");
+            }
             jdbcProps.setProperty("useSSL", "false");
+            jdbcProps.setProperty("requireSSL", "false");
+            jdbcProps.setProperty("verifyServerCertificate", "false");
+            jdbcProps.setProperty("enabledTLSProtocols", "TLSv1,TLSv1.1,TLSv1.2");
+            jdbcProps.setProperty("nullCatalogMeansCurrent", "true");
 
             in = new FileInputStream("/opt/open-xchange/etc/dbconnector.yaml");
             Yaml yaml = new Yaml(new SafeConstructor());
@@ -426,20 +435,30 @@ public class Datamining {
             in = null;
 
             if (Map.class.isInstance(yamlObj)) {
-                Map<String, Object> jdbcConfig = (Map<String, Object>) yamlObj;
-                Object obj = jdbcConfig.get("com.mysql.jdbc");
+                Object obj = ((Map<String, Object>) yamlObj).get("com.mysql.jdbc");
                 if (Map.class.isInstance(obj)) {
-                    Map<String, String> args = ((Map<String, Object>) obj).entrySet().stream().collect(Collectors.toMap(Map.Entry::getKey, e -> e.getValue().toString()));
+                    @SuppressWarnings("unchecked")
+                    Stream<Entry<String, Object>> jdbcPropsStream = ((Map<String, Object>) obj).entrySet().stream();
+                    if (applyTimeout == false) {
+                        // Drop timeout-related JDBC properties
+                        jdbcPropsStream = jdbcPropsStream.filter(jdbcPropEntry -> isNoTimeoutSetting(jdbcPropEntry));
+                    }
+                    Map<String, String> args = jdbcPropsStream.collect(Collectors.toMap(Map.Entry::getKey, e -> e.getValue().toString()));
                     jdbcProps.putAll(args);
-                    System.out.println("JDBC properties were successfully read from /opt/open-xchange/etc/dbconnector.yaml");
+                    System.out.println("JDBC properties successfully read from '/opt/open-xchange/etc/dbconnector.yaml' file");
                 }
             }
 
             jdbcProperties = jdbcProps;
         } catch (FileNotFoundException e) {
-            System.out.println("File /opt/open-xchange/etc/dbconnector.yaml is not available");
+            System.out.println("File '/opt/open-xchange/etc/dbconnector.yaml' is not available");
+            System.out.println("Aborting...");
+            System.exit(1);
         } catch (IOException e) {
-            System.out.println("File /opt/open-xchange/etc/dbconnector.yaml is not readable");
+            System.out.println("File '/opt/open-xchange/etc/dbconnector.yaml' is not readable: " + e.getMessage());
+            e.printStackTrace(System.out);
+            System.out.println("Aborting...");
+            System.exit(1);
         } finally {
             if (null != in) {
                 try {
@@ -451,10 +470,21 @@ public class Datamining {
         }
     }
 
+    /**
+     * Checks if given JDBC property entry is neither <code>"connectTimeout"</code> nor <code>"socketTimeout"</code>.
+     *
+     * @param jdbcPropEntry The JDBC property entry to check
+     * @return <code>true</code> if neither <code>"connectTimeout"</code> nor <code>"socketTimeout"</code>; otherwise false
+     */
+    private static boolean isNoTimeoutSetting(Map.Entry<String, Object> jdbcPropEntry) {
+        String propertyName = jdbcPropEntry.getKey();
+        return ("connectTimeout".equals(propertyName) || "socketTimeout".equals(propertyName)) == false;
+    }
+
     protected static void report(String parameter, String value) {
-        String combined = parameter + "=" + value;
+        String combined = new StringBuilder(parameter).append('=').append(value).toString();
         allTheAnswers.put(parameter, value);
-        reportStringBuilder.append(combined + "\n");
+        reportStringBuilder.append(combined + '\n');
         if (verbose) {
             System.out.println(combined);
         }
@@ -540,9 +570,8 @@ public class Datamining {
         }
     }
 
-    private static ArrayList<Schema> getAllSchemata() {
+    private static List<Schema> getAllSchemata(Connection configdbConnection) {
         allTheQuestions.add(NUMBER_OF_SCHEMATA);
-        ArrayList<Schema> schemata = new ArrayList<Schema>();
         try {
             if (configdbConnection != null) {
                 report("configdbUrl", configDBURL);
@@ -554,7 +583,12 @@ public class Datamining {
                     String sql = "SELECT DISTINCT csp.db_schema, csp.read_db_pool_id, dp.url, dp.login, dp.password FROM context_server2db_pool csp, db_pool dp WHERE csp.read_db_pool_id = dp.db_pool_id;";
                     result = query.executeQuery(sql);
 
-                    while (result.next()) {
+                    if (result.next() == false) {
+                        return Collections.emptyList();
+                    }
+
+                    List<Schema> schemata = new ArrayList<Schema>();
+                    do {
                         Schema schema = new Schema(
                             result.getString("db_schema"),
                             result.getString("read_db_pool_id"),
@@ -562,7 +596,8 @@ public class Datamining {
                             result.getString("login"),
                             result.getString("password"));
                         schemata.add(schema);
-                    }
+                    } while (result.next());
+                    return schemata;
                 } catch (SQLException e) {
                     e.printStackTrace();
                 } finally {
@@ -572,12 +607,11 @@ public class Datamining {
         } catch (Exception e) {
             e.printStackTrace();
         }
-        return schemata;
+        return Collections.emptyList();
     }
 
-    private static ArrayList<String> reportAverageFilestoreSize() {
+    private static List<String> reportAverageFilestoreSize(Connection configdbConnection) {
         allTheQuestions.add(AVERAGE_FILESTORE_SIZE);
-        ArrayList<String> schemata = new ArrayList<String>();
         try {
             if (configdbConnection != null) {
                 Statement query = null;
@@ -588,10 +622,16 @@ public class Datamining {
                     String sql = "SELECT ROUND(AVG(size)) FROM filestore";
                     result = query.executeQuery(sql);
 
-                    while (result.next()) {
-                        String size = result.getString(1);
-                        report(AVERAGE_FILESTORE_SIZE, size);
+                    if (result.next() == false) {
+                        return Collections.emptyList();
                     }
+
+                    List<String> schemata = new ArrayList<String>();
+                    do {
+                        String size = result.getString(1);
+                        report(AVERAGE_FILESTORE_SIZE, result.wasNull() ? "0" : size);
+                    } while (result.next());
+                    return schemata;
                 } catch (SQLException e) {
                     e.printStackTrace();
                 } finally {
@@ -599,9 +639,9 @@ public class Datamining {
                 }
             }
         } catch (Exception e) {
-            //TODO why no exception handling here? Can be ignored?
+            e.printStackTrace();
         }
-        return schemata;
+        return Collections.emptyList();
     }
 
     private static void reportAverageNumberOfInfostoreObjectsPerContext() {
@@ -622,7 +662,7 @@ public class Datamining {
 
                         while (result.next()) {
                             String numberInOneSchema = result.getString(1);
-                            numberInAllSchemata += Float.parseFloat(numberInOneSchema);
+                            numberInAllSchemata += (result.wasNull() ? 0.0F : Float.parseFloat(numberInOneSchema));
                         }
                         conn.close();
                     } catch (SQLException e) {
@@ -633,7 +673,7 @@ public class Datamining {
                 }
             }
         } catch (Exception e) {
-            //TODO why no exception handling here? Can be ignored?
+            e.printStackTrace();
         }
         report(AVERAGE_NUMBER_OF_INFOSTORE_OBJECTS_PER_CONTEXT, Float.toString(numberInAllSchemata / allSchemata.size()));
     }
@@ -656,7 +696,7 @@ public class Datamining {
 
                         while (result.next()) {
                             String numberInOneSchema = result.getString(1);
-                            numberInAllSchemata += Float.parseFloat(numberInOneSchema);
+                            numberInAllSchemata += (result.wasNull() ? 0.0F : Float.parseFloat(numberInOneSchema));
                         }
                         conn.close();
                     } catch (SQLException e) {
@@ -667,13 +707,13 @@ public class Datamining {
                 }
             }
         } catch (Exception e) {
-            //TODO why no exception handling here? Can be ignored?
+            e.printStackTrace();
         }
         report(AVERAGE_NUMBER_OF_INFOSTORE_OBJECTS_PER_SCHEMA, Float.toString(numberInAllSchemata / allSchemata.size()));
     }
 
     protected static BigInteger countOverAllSchemata(String sql) {
-        BigInteger numberOfObjects = new BigInteger("0");
+        BigInteger numberOfObjects = BigInteger.valueOf(0);
         for (Schema schema : allSchemata) {
             String url = generateDbUrlFrom(schema);
             MySQLConnection conn = Datamining.getDBConnection(url, schema.getLogin(), schema.getPassword());
@@ -685,7 +725,7 @@ public class Datamining {
                     result = query.executeQuery(sql);
                     while (result.next()) {
                         String numberInOneSchema = result.getString(1);
-                        numberOfObjects = numberOfObjects.add(new BigInteger(numberInOneSchema));
+                        numberOfObjects = numberOfObjects.add(result.wasNull() ? BigInteger.valueOf(0) : new BigInteger(numberInOneSchema));
                     }
                 } catch (SQLException e) {
                     e.printStackTrace();
@@ -758,14 +798,14 @@ public class Datamining {
                 ResultSet result = null;
                 try {
                     // GROUP BY CLAUSE: ensure ONLY_FULL_GROUP_BY compatibility
-                    prep = conn.prepareStatement("SELECT COUNT(id) FROM user_mail_account WHERE name != ? GROUP BY cid,user");
-                    prep.setString(1, "E-Mail");
+                    prep = conn.prepareStatement("SELECT COUNT(id) FROM user_mail_account WHERE id != 0 GROUP BY cid,user");
                     result = prep.executeQuery();
                     while (result.next()) {
                         int ecnt = result.getInt(1);
                         ecnt = ecnt > max ? max : ecnt;
                         Integer ecntI = Integer.valueOf(ecnt);
-                        if (null != ret.get(ecntI)) {
+                        Integer count = ret.get(ecntI);
+                        if (null != count) {
                             ret.put(ecntI, Integer.valueOf(ret.get(ecntI).intValue() + 1));
                         } else {
                             ret.put(ecntI, Integer.valueOf(1));
@@ -782,7 +822,7 @@ public class Datamining {
     }
 
     protected static BigInteger maximumForAllSchemata(String sql) {
-        BigInteger numberOfObjects = new BigInteger("0");
+        BigInteger numberOfObjects = BigInteger.valueOf(0L);
         for (Schema schema : allSchemata) {
             String url = generateDbUrlFrom(schema);
             MySQLConnection conn = Datamining.getDBConnection(url, schema.getLogin(), schema.getPassword());
@@ -794,9 +834,9 @@ public class Datamining {
                     result = query.executeQuery(sql);
                     while (result.next()) {
                         String numberInOneSchema = result.getString(1);
-                        BigInteger numberInThisSchema = new BigInteger(numberInOneSchema);
-                        if (numberInThisSchema.compareTo(numberOfObjects) == 1){
-                        	numberOfObjects = numberInThisSchema;
+                        BigInteger numberInThisSchema = result.wasNull() ? BigInteger.valueOf(0L) : new BigInteger(numberInOneSchema);
+                        if (numberInThisSchema.compareTo(numberOfObjects) == 1) {
+                            numberOfObjects = numberInThisSchema;
                         }
                     }
                 } catch (SQLException e) {
@@ -822,7 +862,7 @@ public class Datamining {
                     result = query.executeQuery(sql);
                     while (result.next()) {
                         String numberInOneSchema = result.getString(1);
-                        float numberInThisSchema = Float.parseFloat(numberInOneSchema);
+                        float numberInThisSchema = result.wasNull() ? 0.0F : Float.parseFloat(numberInOneSchema);
                         numberOfObjects = numberOfObjects + numberInThisSchema;
                     }
                 } catch (SQLException e) {
