@@ -1,54 +1,28 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
 package com.openexchange.datamining;
 
+import static com.openexchange.java.Autoboxing.f;
+import static com.openexchange.java.Autoboxing.i;
 import java.math.BigInteger;
 import java.util.HashMap;
 import java.util.LinkedHashMap;
@@ -204,6 +178,7 @@ public class Questions {
     private static void handleError(Exception e) {
         //TODO properly handle errors
         System.out.println("Error : " + e.getMessage());
+        e.printStackTrace(System.out);
     }
 
     protected static void reportNumberOfUsersWithEventsInPrivateCalendarThatAreInTheFutureAndAreNotYearlySeries() {
@@ -342,7 +317,7 @@ public class Questions {
         try {
             Datamining.allTheQuestions.add(MAXIMUM_NUMBER_OF_CREATED_CONTACTS_FOR_ONE_USER);
             // GROUP BY CLAUSE: ensure ONLY_FULL_GROUP_BY compatibility
-            String sql = "SELECT MAX(count) FROM (SELECT cid, created_from, count(*) AS count FROM prg_contacts WHERE userid IS NULL GROUP BY cid, created_from) AS x;";
+            String sql = "SELECT MAX(count) FROM (SELECT count(*) AS count FROM prg_contacts WHERE userid IS NULL GROUP BY cid, created_from) AS x;";
             BigInteger numberOfInfostoreObjects = Datamining.maximumForAllSchemata(sql);
             Datamining.report(MAXIMUM_NUMBER_OF_CREATED_CONTACTS_FOR_ONE_USER, numberOfInfostoreObjects.toString());
         } catch (Exception e) {
@@ -365,7 +340,7 @@ public class Questions {
     protected static void reportMaximumNumberOfCreatedDocumentsForOneUser() {
         try {
             Datamining.allTheQuestions.add(MAXIMUM_NUMBER_OF_CREATED_DOCUMENTS_FOR_ONE_USER);
-            String sql = "SELECT MAX(count) FROM (SELECT cid, created_by, count(*) AS count FROM infostore_document) AS x;";
+            String sql = "SELECT MAX(count) FROM (SELECT count(version_number) AS count FROM infostore_document GROUP BY cid, created_by) AS x;";
             BigInteger numberOfInfostoreObjects = Datamining.maximumForAllSchemata(sql);
             Datamining.report(MAXIMUM_NUMBER_OF_CREATED_DOCUMENTS_FOR_ONE_USER, numberOfInfostoreObjects.toString());
         } catch (Exception e) {
@@ -376,7 +351,7 @@ public class Questions {
     protected static void reportMaximumNumberOfCreatedTasksForOneUser() {
         try {
             Datamining.allTheQuestions.add(MAXIMUM_NUMBER_OF_CREATED_TASKS_FOR_ONE_USER);
-            String sql = "SELECT MAX(count) FROM (SELECT cid, created_from, count(*) AS count FROM task) AS x;";
+            String sql = "SELECT MAX(count) FROM (SELECT count(id) AS count FROM task GROUP BY cid, created_from) AS x;";
             BigInteger numberOfInfostoreObjects = Datamining.maximumForAllSchemata(sql);
             Datamining.report(MAXIMUM_NUMBER_OF_CREATED_TASKS_FOR_ONE_USER, numberOfInfostoreObjects.toString());
         } catch (Exception e) {
@@ -388,7 +363,7 @@ public class Questions {
         try {
             Datamining.allTheQuestions.add(MAXIMUM_NUMBER_OF_CREATED_CONTACTS_FOR_ONE_USER);
             // GROUP BY CLAUSE: ensure ONLY_FULL_GROUP_BY compatibility
-            String sql = "SELECT MAX(count) FROM (SELECT cid, created_from, count(*) AS count FROM prg_contacts WHERE userid IS NULL AND field02 IS NOT NULL AND field03 IS NOT NULL GROUP BY cid, created_from) AS x;";
+            String sql = "SELECT MAX(count) FROM (SELECT count(*) AS count FROM prg_contacts WHERE userid IS NULL AND field02 IS NOT NULL AND field03 IS NOT NULL GROUP BY cid, created_from) AS x;";
             BigInteger numberOfInfostoreObjects = Datamining.maximumForAllSchemata(sql);
             Datamining.report(MAXIMUM_NUMBER_OF_CREATED_CONTACTS_FOR_ONE_USER, numberOfInfostoreObjects.toString());
         } catch (Exception e) {
@@ -469,8 +444,8 @@ public class Questions {
             LinkedHashMap<Integer, Integer> dms = Datamining.draftMailOverAllSchemata(new int[] { 5000, 10000, 500000 });
             int ll = 1;
             for (final Entry<Integer, Integer> entry : dms.entrySet()) {
-                Datamining.report("draftMailSizeBetween" + Tools.humanReadableBytes("" + ll) + "And" + Tools.humanReadableBytes("" + entry.getKey()), Integer.toString(entry.getValue() == null ? 0 : entry.getValue()));
-                ll = entry.getKey();
+                Datamining.report("draftMailSizeBetween" + Tools.humanReadableBytes("" + ll) + "And" + Tools.humanReadableBytes("" + entry.getKey()), Integer.toString(entry.getValue() == null ? 0 : i(entry.getValue())));
+                ll = i(entry.getKey());
             }
         } catch (Exception e) {
             handleError(e);
@@ -482,7 +457,7 @@ public class Questions {
             int MAXEXT = 5;
             HashMap<Integer, Integer> eaos = Datamining.externalAccountsOverAllSchemata(MAXEXT);
             for (final Entry<Integer, Integer> entry : eaos.entrySet()) {
-                Datamining.report("usersHaving" + (entry.getKey() < MAXEXT ? entry.getKey() : "MoreOrEqual" + entry.getKey()) + "ExternalAccounts", Integer.toString(entry.getValue() == null ? 0 : entry.getValue()));
+                Datamining.report("usersHaving" + (i(entry.getKey()) < MAXEXT ? entry.getKey() : "MoreOrEqual" + entry.getKey()) + "ExternalAccounts", Integer.toString(entry.getValue() == null ? 0 : i(entry.getValue())));
             }
         } catch (Exception e) {
             handleError(e);
@@ -495,7 +470,7 @@ public class Questions {
 
             String sql = "SELECT AVG(file_size) FROM infostore_document;";
             Float result = Datamining.averageForAllSchemata(sql);
-            int resultInt = Math.round(result);
+            int resultInt = Math.round(f(result));
 
             Datamining.report(AVERAGE_DOCUMENT_SIZE, Tools.humanReadableBytes(Integer.toString(resultInt)));
         } catch (Exception e) {
@@ -510,8 +485,8 @@ public class Questions {
             // take numberOfUsers and subtract numberOfContexts (thereby not counting the context-admins) -> relevant number of users
             // divide the two (numberOfContacts / (numberOfUsers - numberOfContexts)) -> average number of contacts per User
 
-            float numberOfContacts = Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_CONTACTS));
-            float numberOfUsers = Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USERS_WHO_HAVE_CONTACTS));
+            float numberOfContacts = f(Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_CONTACTS)));
+            float numberOfUsers = f(Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USERS_WHO_HAVE_CONTACTS)));
 
             Datamining.report(AVERAGE_NUMBER_OF_CONTACTS_PER_USER_WHO_HAS_CONTACTS_AT_ALL, Float.toString(numberOfContacts / numberOfUsers));
         } catch (Exception e) {
@@ -526,8 +501,8 @@ public class Questions {
             // take numberOfUsers and subtract numberOfContexts (thereby not counting the context-admins) -> relevant number of users
             // divide the two (numberOfContacts / (numberOfUsers - numberOfContexts)) -> average number of contacts per User
 
-            float numberOfContacts = Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USER_CREATED_CONTACTS));
-            float numberOfUsers = Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USERS_WHO_CREATED_CONTACTS));
+            float numberOfContacts = f(Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USER_CREATED_CONTACTS)));
+            float numberOfUsers = f(Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USERS_WHO_CREATED_CONTACTS)));
 
             Datamining.report(AVERAGE_NUMBER_OF_CONTACTS_PER_USER_WHO_HAS_CREATED_CONTACTS, Float.toString(numberOfContacts / numberOfUsers));
         } catch (Exception e) {
@@ -539,8 +514,8 @@ public class Questions {
         try {
             Datamining.allTheQuestions.add(AVERAGE_NUMBER_OF_APPOINTMENTS_PER_USER_WHO_HAS_APPOINTMENTS_AT_ALL);
 
-            float numberOfAppointments = Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_APPOINTMENTS));
-            float numberOfUsers = Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USERS_WHO_CREATED_APPOINTMENTS));
+            float numberOfAppointments = f(Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_APPOINTMENTS)));
+            float numberOfUsers = f(Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USERS_WHO_CREATED_APPOINTMENTS)));
 
             Datamining.report(AVERAGE_NUMBER_OF_APPOINTMENTS_PER_USER_WHO_HAS_APPOINTMENTS_AT_ALL, Float.toString(numberOfAppointments / numberOfUsers));
         } catch (Exception e) {
@@ -552,8 +527,8 @@ public class Questions {
         try {
             Datamining.allTheQuestions.add(AVERAGE_NUMBER_OF_TASKS_PER_USER_WHO_HAS_TASKS_AT_ALL);
 
-            float numberOfTasks = Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_TASKS));
-            float numberOfUsers = Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USERS_WHO_CREATED_TASKS));
+            float numberOfTasks = f(Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_TASKS)));
+            float numberOfUsers = f(Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USERS_WHO_CREATED_TASKS)));
 
             Datamining.report(AVERAGE_NUMBER_OF_TASKS_PER_USER_WHO_HAS_TASKS_AT_ALL, Float.toString(numberOfTasks / numberOfUsers));
         } catch (Exception e) {
@@ -565,8 +540,8 @@ public class Questions {
         try {
             Datamining.allTheQuestions.add(AVERAGE_NUMBER_OF_DOCUMENTS_PER_USER_WHO_HAS_DOCUMENTS_AT_ALL);
 
-            float numberOfDocuments = Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_DOCUMENTS));
-            float numberOfUsers = Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USERS_WHO_CREATED_DOCUMENTS));
+            float numberOfDocuments = f(Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_DOCUMENTS)));
+            float numberOfUsers = f(Float.valueOf(Datamining.getOneAnswer(NUMBER_OF_USERS_WHO_CREATED_DOCUMENTS)));
 
             Datamining.report(AVERAGE_NUMBER_OF_DOCUMENTS_PER_USER_WHO_HAS_DOCUMENTS_AT_ALL, Float.toString(numberOfDocuments / numberOfUsers));
         } catch (Exception e) {
